# Generated by roxygen2: do not edit by hand

export(detect_all_events)
export(detect_between_maxima)
export(detect_hyperglycemic_events)
export(detect_hypoglycemic_events)
export(excursion)
export(find_local_maxima)
export(find_max_after_hours)
export(find_max_before_hours)
export(find_min_after_hours)
export(find_min_before_hours)
export(find_new_maxima)
export(grid)
export(interpolate_cgm)
export(maxima_grid)
export(mod_grid)
export(orderfast)
export(sensor_wear)
export(start_finder)
export(transform_df)
importFrom(Rcpp,evalCpp)
useDynLib(cgmguru, .registration = TRUE)
