---
title: "find_new_maxima function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{find_new_maxima function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# find_new_maxima

Runs the documented examples for `find_new_maxima()`.

```{r}
example(find_new_maxima, package = "cgmguru", run.dontrun = FALSE)
```


