---
title: "orderfast function"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{orderfast function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", warning = FALSE, message = FALSE)
library(cgmguru)
library(iglu)
```

# orderfast

## Overview
Fast ordering of a dataframe by `id` and `time`, optimized for large CGM datasets. Returns the input sorted while preserving columns.

## Inputs
- **df**: Dataframe containing `id` and `time`

## Returns
- Dataframe ordered by `id`, then `time`

## Run documented examples
```{r}
example(orderfast, package = "cgmguru", run.dontrun = FALSE)
```


